-----------------------------------------------------------------------------------------------
-- Client Lua Script for DecorKeybinds
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "HousingLib"
 
-----------------------------------------------------------------------------------------------
-- DecorKeybinds Module Definition, Millek Lightfoot
-----------------------------------------------------------------------------------------------
local DecorKeybinds = {} 

local pi = 3.14159265

local ktKeyChar = {
	[49] = "1",
	[50] = "2",
	[51] = "3",
	[52] = "4",
	[53] = "5", 
	[54] = "6",
	[55] = "7", 
	[56] = "8",
	[57] = "9",
	[48] = "0",
	
	[81] = "Q",
	[87] = "W",
	[69] = "E",
	[82] = "R",
	[84] = "T",
	[89] = "Y",
	[85] = "U",
	[73] = "I",
	[79] = "O",
	[80] = "P",
	
	[65] = "A",
	[83] = "S",
	[68] = "D",
	[70] = "F",
	[71] = "G",
	[72] = "H",
	[74] = "J",
	[75] = "K",
	[76] = "L",
	
	[90] = "Z",
	[88] = "X",
	[67] = "C",
	[86] = "V",
	[66] = "B",
	[78] = "N",
	[77] = "M",
	
	[33] = "Page Up",
	[34] = "Page Down",
	[35] = "End",
	[36] = "Home",
	[45] = "Ins",
	[46] = "Del",
	
	[38] = "Up",
	[37] = "Left",
	[40] = "Down",
	[39] = "Right",
	
	--numpad
	[96] = "num0",
	[97] = "num1",
	[98] = "num2",
	[99] = "num3",
	[100] = "num4",
	[101] = "num5",
	[102] = "num6",
	[103] = "num7",
	[104] = "num8",
	[105] = "num9",
	[144] = "NumLock",
	[111] = "num/",
	[106] = "num*",
	[109] = "num-",
	[107] = "num+",
	[110] = ".numDel",
	[13]  = "Enter",
	
	[192] = "~`",
	[9]   = "Tab",
	[20]  = "CapsL",
	[16] = "Shift",
	[17] = "Ctrl",
	[32] = "Space",
	[220] = "\\",
	[8] = "BackSp",
	
	[188] = ",<",
	[190] = ".>",
	[191] = "/?",
	[186] = ":",
	[222] = "'",
	[219] = "[",
	[221] = "]",
	[189] = "-",
	[187] = "=",
	
	[112] = "F1",
	[113] = "F2",
	[114] = "F3",
	[115] = "F4",
	[116] = "F5",
	[117] = "F6",
	[118] = "F7",
	[119] = "F8",
	[120] = "F9",
	[121] = "F10",
	[122] = "F11",
	[123] = "F12",	
}

------------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function DecorKeybinds:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 


    return o
end

function DecorKeybinds:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	
end
 

-----------------------------------------------------------------------------------------------
-- DecorKeybinds OnLoad
-----------------------------------------------------------------------------------------------
function DecorKeybinds:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("DecorKeybinds.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	self.wndMain = Apollo.LoadForm("DecorKeybinds.xml", "DecorKeybindsForm", nil, self)
	self.wndKeyBinds = self.wndMain:FindChild("KeyBindWnd")
	self.wndMoveSlider = self.wndMain:FindChild("MoveSliderBar")
	self.wndRotSlider = self.wndMain:FindChild("RotationSliderBar")
	
	self.wndMain:ToFront()
		
	-- Keybinding Variables and settings
	self.tDefaultSettings = {}
	self.tDefaultSettings.X = 103
	self.tDefaultSettings.Y = 104
	self.tDefaultSettings.Z = 105
	self.tDefaultSettings.Pos = 100
	self.tDefaultSettings.Neg = 102
	self.tDefaultSettings.R = 97
	self.tDefaultSettings.S = 98
	self.tDefaultSettings.T = 99
	
	self.tSavedBtns = {}
	self.tSavedBtns.X = 0
	self.tSavedBtns.Y = 0
	self.tSavedBtns.Z = 0
	self.tSavedBtns.Pos = 0
	self.tSavedBtns.Neg = 0
	self.tSavedBtns.R = 0
	self.tSavedBtns.S = 0
	self.tSavedBtns.T = 0

	self.tSettingsWnds = {}
	self.tSettingsWnds.X = self.wndMain:FindChild("XAxisBind")
	self.tSettingsWnds.Y = self.wndMain:FindChild("YAxisBind")
	self.tSettingsWnds.Z = self.wndMain:FindChild("ZAxisBind")
	self.tSettingsWnds.Pos = self.wndMain:FindChild("PosMoveBind")
	self.tSettingsWnds.Neg = self.wndMain:FindChild("NegMoveBind")
	self.tSettingsWnds.R = self.wndMain:FindChild("RotationBind")
	self.tSettingsWnds.S = self.wndMain:FindChild("ScaleBind")
	self.tSettingsWnds.T = self.wndMain:FindChild("TranslationBind")
	
	self.tAssignBtns = {}
	self.tAssignBtns.X = false
	self.tAssignBtns.Y = false
	self.tAssignBtns.Z = false
	self.tAssignBtns.Pos = false
	self.tAssignBtns.Neg = false
	self.tAssignBtns.R = false
	self.tAssignBtns.S = false
	self.tAssignBtns.T = false
	
	self.tMode = {}
	self.tMode.X = true --x axis mode
	self.tMode.Y = false -- y axis
	self.tMode.Z = false -- z axis
	self.tMode.R = false -- rotation mode
	self.tMode.S = false -- scale
	self.tMode.T = true -- translation
	
	self.AcceptKeyInput = false
	
	--Housing Decor settings
	self.nItemHandle = 0
	self.HousingAddon = Apollo.GetAddon("Housing")
	self.bPos = true
	self.bRot = true
	self.bScale = true
	
	self.Move = .05
	--self.Rotation = 0.785398163
	self.RotationMod = 2

end

-----------------------------------------------------------------------------------------------
-- DecorKeybinds OnDocLoaded
-----------------------------------------------------------------------------------------------
function DecorKeybinds:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "DecorKeybindsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("dk", "OnDecorKeybindsOn", self)
		Apollo.RegisterEventHandler("SystemKeyDown", "KeyDown", self)

		-- Do additional Addon initialization here
		Apollo.RegisterTimerHandler("DecorPlacementTimer", 	"OnCloneTimer", self)
		Apollo.CreateTimer("DecorPlacementTimer", 0.3, false)
	    Apollo.StopTimer("DecorPlacementTimer")	end
	
		Apollo.RegisterEventHandler("HousingPanelControlOpen", "OnEnterHousing", self)
		Apollo.RegisterEventHandler("HousingPanelControlClose", "OnLeaveHousing", self)
			
		Print("Decor Keybinds loaded, Type /dk to start!")
	end
-----------------------------------------------------------------------------------------------
-- DecorKeybinds Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function DecorKeybinds:OnEnterHousing(wndHandler, wndControl)
	if HousingLib.IsHousingWorld() then
		--Entered Housing, register for KeyDown event
		Apollo.RegisterEventHandler("SystemKeyDown", "KeyDown", self)
		Print("Type /dk to start Decor Keybinds, You have entered a Housing plot!")	end
end

function DecorKeybinds:OnLeaveHousing(wndHandler, wndControl)
	if HousingLib.IsHousingWorld() then
		--Left Housing, Unregister for KeyDown event
		Apollo.RemoveEventHandler("SystemKeyDown", self)
		Print("Decor Keybind Events Unregistered, you have left a housing plot")
	end
end

-- on SlashCommand "/dk"
function DecorKeybinds:OnDecorKeybindsOn()
	self.wndMain:Invoke() -- show the window
	
	local SaveBtns = {}
	SaveBtns.X = 0
	SaveBtns.Y = 0
	SaveBtns.Z = 0
	SaveBtns.Pos = 0
	SaveBtns.Neg = 0
	SaveBtns.R = 0
	SaveBtns.S = 0
	SaveBtns.T = 0

	local Wnds = {}
	Wnds.X = self.wndMain:FindChild("XAxisBind")
	Wnds.Y = self.wndMain:FindChild("YAxisBind")
	Wnds.Z = self.wndMain:FindChild("ZAxisBind")
	Wnds.Pos = self.wndMain:FindChild("PosMoveBind")
	Wnds.Neg = self.wndMain:FindChild("NegMoveBind")
	Wnds.R = self.wndMain:FindChild("RotationBind")
	Wnds.S = self.wndMain:FindChild("ScaleBind")
	Wnds.T = self.wndMain:FindChild("TranslationBind")

	--defualt settings
	for k,v in pairs(self.tDefaultSettings) do
		SaveBtns[k] = v
		self.tSavedBtns[k] = SaveBtns[k]
		local wnd = Wnds[k]
		wnd:SetText(ktKeyChar[v])
	end 

	--check to see if we have saved settings
	if self.tSavedData ~= nil then
		for k,v in pairs(self.tSavedData) do
			SaveBtns[k] = v
			self.tSavedBtns[k] = SaveBtns[k]
			local wnd  = Wnds[k]
			wnd:SetText(ktKeyChar[v])
		end
	else Print("NO Saved Data")
	end
end


-----------------------------------------------------------------------------------------------
-- DecorKeybindsForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function DecorKeybinds:OnCancel()
	self.wndMain:Close() -- hide the window
end

-- restore Game Action Input after pausing
function DecorKeybinds:RestoreAfterPause()
	GameLib.PauseGameActionInput(false)
end

function DecorKeybinds:OnDefaultBtn(wndHandler, wndControl, eMouseButton)
	local SaveBtns = {}
	SaveBtns.X = 0
	SaveBtns.Y = 0
	SaveBtns.Z = 0
	SaveBtns.Pos = 0
	SaveBtns.Neg = 0
	SaveBtns.R = 0
	SaveBtns.S = 0
	SaveBtns.T = 0

	local Wnds = {}
	Wnds.X = self.wndMain:FindChild("XAxisBind")
	Wnds.Y = self.wndMain:FindChild("YAxisBind")
	Wnds.Z = self.wndMain:FindChild("ZAxisBind")
	Wnds.Pos = self.wndMain:FindChild("PosMoveBind")
	Wnds.Neg = self.wndMain:FindChild("NegMoveBind")
	Wnds.R = self.wndMain:FindChild("RotationBind")
	Wnds.S = self.wndMain:FindChild("ScaleBind")
	Wnds.T = self.wndMain:FindChild("TranslationBind")

	--defualt settings
	for k,v in pairs(self.tDefaultSettings) do
		SaveBtns[k] = v
		self.tSavedBtns[k] = SaveBtns[k]
		local wnd = Wnds[k]
		wnd:SetText(ktKeyChar[v])
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------
function DecorKeybinds:SelectKey( wndHandler, wndControl, eMouseButton )-- SelectKey Function  -- Button was pressed to bind a key
	GameLib.PauseGameActionInput(true)--Pause Game Input to bind buttons to decor movement
	
	if     wndControl:GetName() == "XAxisBtn" then -- find which button was pressed
		self.AcceptKeyInput = true
		self.tAssignBtns.X = true
	elseif wndControl:GetName() == "YAxisBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.Y = true
	elseif wndControl:GetName() == "ZAxisBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.Z = true
	elseif wndControl:GetName() == "PosMoveBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.Pos = true
	elseif wndControl:GetName() == "NegMoveBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.Neg = true
	elseif wndControl:GetName() == "RotationBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.R = true
	elseif wndControl:GetName() == "ScaleBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.S = true
	elseif wndControl:GetName() == "TranslationBtn" then
		self.AcceptKeyInput = true
		self.tAssignBtns.T = true
	end	
end
--------------------------------------------------------------------------------------------------------------------------------------------

function DecorKeybinds:KeyDown(Key)	-- KeyDown Function called on SystemKeyDown
--------Setting up Keybinds here
	
	local bAcceptKey = false
	local bKeySaved = false
	
	if self.AcceptKeyInput == true then 
		--Button was pressed to bind a key
		for k, AssignBtn in pairs(self.tAssignBtns) do
			if AssignBtn == true then
				--Print("key is " .. Key .. " ," .. tostring(AssignBtn))
				for i, v in pairs(self.tSavedBtns) do 
					if v == Key then 
						bAcceptKey = false
						Print("Key already used, try again")
						return
					else 
						bAcceptKey = true
					end
				end
			end
			if tostring(k) == "X" and bAcceptKey == true then
				self.tSavedBtns.X = Key
				Print(Key .. " ," .. self.tSavedBtns.X)
				self.wndMain:FindChild("XAxisBind"):SetText(ktKeyChar[self.tSavedBtns.X])
				self.tAssignBtns.X = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
			    break
			elseif tostring(k) == "Y" and bAcceptKey == true then
				self.tSavedBtns.Y = Key
				Print(k .. " ," .. self.tSavedBtns.Y)
				self.wndMain:FindChild("YAxisBind"):SetText(ktKeyChar[self.tSavedBtns.Y])
				self.tAssignBtns.Y = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			elseif tostring(k) == "Z" and bAcceptKey == true then
				self.tSavedBtns.Z = Key
				Print(k .. " ," .. self.tSavedBtns.Z)
				self.wndMain:FindChild("ZAxisBind"):SetText(ktKeyChar[self.tSavedBtns.Z])
				self.tAssignBtns.Z = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			elseif tostring(k) == "Pos" and bAcceptKey == true then
				self.tSavedBtns.Pos = Key
				Print(k .. " ," .. self.tSavedBtns.Pos)
				self.wndMain:FindChild("PosMoveBind"):SetText(ktKeyChar[self.tSavedBtns.Pos])
				self.tAssignBtns.Pos = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			elseif tostring(k) == "Neg" and bAcceptKey == true then
				self.tSavedBtns.Neg = Key
				Print(k .. " ," .. self.tSavedBtns.Neg)
				self.wndMain:FindChild("NegMoveBind"):SetText(ktKeyChar[self.tSavedBtns.Neg])
				self.tAssignBtns.Neg = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			elseif tostring(k) == "R" and bAcceptKey == true then
				self.tSavedBtns.R = Key
				Print(k .. " ," .. self.tSavedBtns.R)
				self.wndMain:FindChild("RotationBind"):SetText(ktKeyChar[self.tSavedBtns.R])
				self.tAssignBtns.R = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			elseif tostring(k) == "S" and bAcceptKey == true then
				self.tSavedBtns.S = Key
				Print(k .. " ," .. self.tSavedBtns.S)
				self.wndMain:FindChild("ScaleBind"):SetText(ktKeyChar[self.tSavedBtns.S])
				self.tAssignBtns.S = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			elseif tostring(k) == "T" and bAcceptKey == true then
				self.tSavedBtns.T = Key
				Print(k .. " ," .. self.tSavedBtns.T)
				self.wndMain:FindChild("TranslationBind"):SetText(ktKeyChar[self.tSavedBtns.T])
				self.tAssignBtns.T = false
				bKeySaved = true
				Apollo.CreateTimer("RestoreAfterPause", .1, false)
				self.AcceptKeyInput = false
				break
			end				
		end -- closes for loop
		if bKeySaved == false then Print("Key not Saved -- error") end
	end -- closes accept key if
	
	--Set the Axis  to move on if not keybinding
	if Key == self.tSavedBtns.X then
		self.tMode.X = true
		self.tMode.Y = false
		self.tMode.Z = false
		Print("X Axis selected")
	elseif Key == self.tSavedBtns.Y then
		self.tMode.X = false
		self.tMode.Y = true
		self.tMode.Z = false
		Print("Y Axis selected")
	elseif Key == self.tSavedBtns.Z then
		self.tMode.X = false
		self.tMode.Y = false
		self.tMode.Z = true
		Print("Z Axis selected")
	elseif Key == self.tSavedBtns.R then
		self.tMode.R = true
		self.tMode.S = false
		self.tMode.T = false
		Print("Rotation Mode selected")
	elseif Key == self.tSavedBtns.S then
		self.tMode.R = false
		self.tMode.S = true
		self.tMode.T = false
		Print("Scale Mode selected")
	elseif Key == self.tSavedBtns.T then
		self.tMode.R = false
		self.tMode.S = false
		self.tMode.T = true
		Print("TranslationMode selected")
	end		
					
--------------------------------------------------------------------------------------------------------------
--Key Input for decor translation
--------------------------------------------------------------------------------------------------------------	
--Put Decor movement code here
	local nDecorHandle = 0
	local wndMove = self.wndMain:FindChild("MoveBox")
	local wndRot = self.wndMain:FindChild("RotationSliderBox")
	self.Move = wndMove:GetText()
	self.RotationMod = wndRot:GetText()
	
	if self.HousingAddon.nPreviewDecorHandle ~= 0 then
		nDecorHandle = self.HousingAddon.nPreviewDecorHandle
		--Print("Preview Decor Handle")
	end
	if self.HousingAddon.nFreePlaceDecorHandle ~= 0 then
		nDecorHandle = self.HousingAddon.nFreePlaceDecorHandle
		--Print("Free Place Decor Handle")
	end
	if nDecorHandle == 0 then
		--Print("No decor selected")
		return
	end
	
	local tDecorInfo = HousingLib.GetDecorIconInfo(nDecorHandle)
	local rads = self.RotationMod*pi/180

	if Key == self.tSavedBtns.Pos then
	
		if self.tMode.X == true and self.tMode.T == true then 
			HousingLib.SetDecorPosition(tDecorInfo.fWorldPosX + self.Move,
								    	tDecorInfo.fWorldPosY,
								    	tDecorInfo.fWorldPosZ)
		elseif self.tMode.Y == true and self.tMode.T == true then
			HousingLib.SetDecorPosition(tDecorInfo.fWorldPosX,
								    	tDecorInfo.fWorldPosY + self.Move,
								    	tDecorInfo.fWorldPosZ)
		elseif self.tMode.Z == true and self.tMode.T == true then
			HousingLib.SetDecorPosition(tDecorInfo.fWorldPosX,
								    	tDecorInfo.fWorldPosY,
								    	tDecorInfo.fWorldPosZ + self.Move)
		elseif self.tMode.X == true and self.tMode.R == true then
			HousingLib.SetDecorRotation(tDecorInfo.fPitch + (rads),
										tDecorInfo.fRoll,
										tDecorInfo.fYaw)
		elseif self.tMode.Y == true and self.tMode.R == true then
			HousingLib.SetDecorRotation(tDecorInfo.fPitch,
										tDecorInfo.fRoll + (rads),
										tDecorInfo.fYaw)
		elseif self.tMode.Z == true and self.tMode.R == true then
			HousingLib.SetDecorRotation(tDecorInfo.fPitch,
										tDecorInfo.fRoll,
										tDecorInfo.fYaw + (rads))
		elseif self.tMode.S == true then
			HousingLib.ScaleDecor(tDecorInfo.fScaleCurrent + self.Move)
		else 
			Print("invalid operation")
		end

	elseif Key == self.tSavedBtns.Neg == true then
	
		if self.tMode.X == true and self.tMode.T == true then 
			HousingLib.SetDecorPosition(tDecorInfo.fWorldPosX - self.Move,
								    	tDecorInfo.fWorldPosY,
								    	tDecorInfo.fWorldPosZ)
		elseif self.tMode.Y == true and self.tMode.T == true then
			HousingLib.SetDecorPosition(tDecorInfo.fWorldPosX,
								    	tDecorInfo.fWorldPosY - self.Move,
								    	tDecorInfo.fWorldPosZ)
		elseif self.tMode.Z == true and self.tMode.T == true then
			HousingLib.SetDecorPosition(tDecorInfo.fWorldPosX,
								    	tDecorInfo.fWorldPosY,
								    	tDecorInfo.fWorldPosZ - self.Move)
		elseif self.tMode.X == true and self.tMode.R == true then
			HousingLib.SetDecorRotation(tDecorInfo.fPitch - (rads),
										tDecorInfo.fRoll,
										tDecorInfo.fYaw)
		elseif self.tMode.Y == true and self.tMode.R == true then
			HousingLib.SetDecorRotation(tDecorInfo.fPitch,
										tDecorInfo.fRoll - (rads),
										tDecorInfo.fYaw)
		elseif self.tMode.Z == true and self.tMode.R == true then
			HousingLib.SetDecorRotation(tDecorInfo.fPitch,
										tDecorInfo.fRoll,
										tDecorInfo.fYaw - (rads))
		elseif self.tMode.S == true then
			HousingLib.ScaleDecor(tDecorInfo.fScaleCurrent - self.Move)
		else 
			Print("invalid operation")
		end

	end
end

function DecorKeybinds:OnMoveBoxReturn(wndHandler, wndControl)
	local wnd = self.wndMain:FindChild("MoveSliderBar")
	wndHandler:SetText(string.format("%.2f", wndHandler:GetText()))
	wnd:SetValue(string.format("%.2f", wndHandler:GetText()))
end

function DecorKeybinds:OnRotationSliderBoxReturn(wndHandler, wndControl)
    local wnd = self.wndMain:FindChild("RotationSliderBar")
	wnd:SetValue(string.format("%.2f", wndHandler:GetText()))
end

function DecorKeybinds:OnMoveSliderChanged(wndHandler, wndControl)
	local wnd = self.wndMain:FindChild("MoveBox")
	wnd:SetText(string.format("%.2f", wndHandler:GetValue()))
end

function DecorKeybinds:OnRotationSliderBarChanged(wndHandler, wndControl)
    local wnd = self.wndMain:FindChild("RotationSliderBox")
	wnd:SetText(string.format("%.2f", wndHandler:GetValue()))
end

--Decor Keybinds On Save Function
function DecorKeybinds:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end

    -- create a table to hold our data
    local tSave = {}
    for k,v in pairs(self.tSavedBtns) do
        -- for each keybind in our list save for next time
        tSave[k] = v
    end

    -- return this value and Apollo will save the file!
    return tSave
end

--DecorKeybinds On Restore
function DecorKeybinds:OnRestore(eLevel, tData)
    --store this and use it later
    self.tSavedData = tData
end


-----------------------------------------------------------------------------------------------
-- DecorKeybinds Instance
-----------------------------------------------------------------------------------------------
local DecorKeybindsInst = DecorKeybinds:new()
DecorKeybindsInst:Init()
